#ifndef CLUSTER_MOVES_H
#define CLUSTER_MOVES_H


/*This is a module containing ALL the code related to doing cluster moves,
 * using data augmentation on a sphere.
 *
 * The prefix is CMV_
 */

/*struct clusters: This structure contains all the information about the current partition of 
 *particles in a system.
 *
 * int head_of[] a list containing the index of the particle at the head
 * of the cluster for each particle
 * ll[]: a linked list. the value at the index of a given particle
 * int cluster_of[]: the index of the cluster of each particle
 */

typedef struct cmnstr {

	int *head_of;
	int *ll;

} Mc_clusters;

/*FUNCTION: CMV_init allocates the memeory for the clusters
 *
 * param cluster: cluster struct to be initialised
 * inoput n_particles: the total number of particles in the system
 */

void CMV_init(Mc_clusters *cluster, int n_particles);

/*FUNTION: CMV_destroy: frees the structure*/

void CMV_destroy(Mc_clusters *cluster);

/*Function CMV_get psudo temperature: returns a psudo_temoerature between max and min
 *
 *input max: the mxaimum allwoed psudotemperature
 *input min: the minimum allowed psudo_temperature
 */

double CMV_get_psudo_temp (double max, double min);

/*Function: Get clusters: This function partitions a system of N particles on a sphere into 
*	clusters, in accordance to the algorithm.
*
* 	Input: Mc_state: the current state of the system
*	Input: Mc_cell: the cell list for the system
*	Input: Psudo_temperature: a parameter of the algorithm that determines average cluster size
*	double(*potential) a function which returns the potential felt by two particles
*	separated by a set distance (the first argument).
*	paramsP is the parameter list for the user-defined potential function
*	output: Mc_clusters: the new cluster partition 
*/
void CMV_get_clusters (Mc_state *state, Mc_cell *cell, Mc_clusters *cluster, double psudo_temp,
			int periodic, double *paramsG,
			double (*potential)(double,double*), double paramsP[]);

/* FUNCTION: move_mc takes a whole cluster and makes a monte carlo move suggesting 
 * it's movement . The new
 * position of the centre of the mass of the cluster moves arounnd the sphere
 * by an angle less than delta_cluster.
*  Input head: the head of the cluster to be moved
* 	Input: Mc_state: the current state of the system
*	Input: Mc_cell: the cell list for the system
*	Input MC_cluster: the clustering struct
*	Input delta: the max length of the move (an radians aroud the sphere)
*	Input: radius: the radius of the sphere
*	Input: move_cluster: Function that moves a cluster in the current geometry
*/

int CMV_move_mc (Mc_state *state, Mc_cell *cell, Mc_clusters *cluster, double delta,
			double *radius, double psudo_temp, int periodic, double *paramsG,
			double (*potential)(double,double*), 
			void (*move_cluster)(Mc_state*,Mc_cell*,Mc_clusters*,int,double,double*),
			double paramsP[]);

/* FUNCTION: spin_mc takes a whole cluster and suggests a monte carlo move that spins it as a whole
*  spins it as a whole. The centre of mass does not change.
*  Input head: the head of the cluster to be moved
* 	Input: Mc_state: the current state of the system
*	Input: Mc_cell: the cell list for the system
*	Input MC_cluster: the clustering struct
*	Input delta: the max length of the move (in radians around the COM of the cluster)
*	Input: mspin_cluster: Function that spins a cluster in the current geometry
*/


int CMV_spin_mc (Mc_state *state, Mc_cell *cell, Mc_clusters *cluster, double delta,
			double *radius, double psudo_temp, int periodic, double *paramsG,
			double (*potential)(double,double*),
			void (*spin_cluster)(Mc_state*,Mc_cell*,Mc_clusters*,int,double,double*),
 			double paramsP[]);

/* FUNCTION: move_cluster takes a whole cluster and moves it as a whole. The new
 * position of the centre of the mass of the cluster moves arounnd the sphere
 * by an angle less than delta_cluster.
*  Input head: the head of the cluster to be moved
* 	Input: Mc_state: the current state of the system
*	Input: Mc_cell: the cell list for the system
*	Input MC_cluster: the clustering struct
*	Input delta: the max length of the move (an radians aroud the sphere)
*	Input: radius: the radius of the sphere
*/

void CMV_move_cluster_sphere (Mc_state *state, Mc_cell *cell, Mc_clusters *cluster,int head, 
		double delta,double *paramsG);


/* FUNCTION: spin_cluster takes a whole cluster and spins it as a whole. The centre of mass does not change
*  Input head: the head of the cluster to be moved
* 	Input: Mc_state: the current state of the system
*	Input: Mc_cell: the cell list for the system
*	Input MC_cluster: the clustering struct
*	Input delta: the max length of the move (in radians around the COM of the cluster)
*/


void CMV_spin_cluster_sphere (Mc_state *state, Mc_cell *cell, Mc_clusters *cluster,int head, 
		double delta,double *paramsG);

/* FUNCTION: move_cluster takes a whole cluster and moves it as a whole. The new
 * position of the centre of the mass of the cluster moves arounnd the plane
 * by a distance less than delta_cluster.
*  Input head: the head of the cluster to be moved
* 	Input: Mc_state: the current state of the system
*	Input: Mc_cell: the cell list for the system
*	Input MC_cluster: the clustering struct
*	Input delta: the max length of the move (an radians aroud the sphere)
*	Input: paramsG: the box size of the plane
*/

void CMV_move_cluster_plane (Mc_state *state, Mc_cell *cell, Mc_clusters *cluster,int head, 
		double delta,double *paramsG);

/* FUNCTION: spin_cluster takes a whole cluster and spins it as a whole. The centre of mass does not change
*  Input head: the head of the cluster to be moved
* 	Input: Mc_state: the current state of the system
*	Input: Mc_cell: the cell list for the system
*	Input MC_cluster: the clustering struct
*	Input delta: the max length of the move (in radians around the COM of the cluster)
*/


void CMV_spin_cluster_plane (Mc_state *state, Mc_cell *cell, Mc_clusters *cluster,int head, 
		double delta,double *paramsG);

#endif
