/*This module contains functions for calculating the potential energy felt
 * by various sets of particles. Only pairwise interaction may be handled
 * The form of the potential is defined via input
 *
 *The module prefix is EGY_
 */


#include"mc_math_h.h"
#include"std_h.h"
#include"mc_state_h.h"
#include"cell_list_h.h"
#include"energy_h.h"
#include"geometry_h.h"

/*Function: energy_calc gets the potential energy of a single particle, at coord
 * u,v.
 *
 * Output: type double: The energy (relative to kT) of the test particle
 * Parameters:
 *	Mc_state state
 *	Mc_cell cell: the cell list
 *	double a_u: The u coordinate of the test particle
 *	double v_u: The v coordinate of the test particle
 *	int index: the index of the particle to be measured. If the particle is imaginary
 *	(ie. not in "state"), set index to n_particles+1
 *	int cell_of: the cell occupied by the target particle
 *	double(*potential) a function which returns the potential felt by two particles
 *	separated by a set distance (the first argument).
 *	paramsP is the parameter list for the user-defined potential function
 */

double EGY_energy_calc(Mc_state *state, Mc_cell *cell, double x, double y, double z, 
			int index, int cell_of, int periodic, double *paramsG,
			double (*potential)(double,double*), double paramsP[]) {
	double energy = 0;
	int i;
	int current_cell = cell_of;
	double *x_list = state->x;
	double *y_list = state->y;
	double *z_list = state->z;
	long *ll = cell->ll;
	long **neighbours = cell->neighbours;
	long *head_of = cell->head_of;

	/*get head of cell in which the particles is*/

	i = head_of[ current_cell ];
	while (i != index && i != -1) {
		energy += (*potential)(GEO_distance(x,y,z,x_list[i],y_list[i],z_list[i],
							periodic, paramsG),
					paramsP);
		i = ll[i];
	}
	/*skip one*/
	if (i != -1) {
		i = ll[i];
	}
	while (i != -1) {
		
		energy += (*potential)(GEO_distance(x,y,z,x_list[i],y_list[i],z_list[i],
							periodic, paramsG),
					paramsP);
		i = ll[i];	
	}

	/*do neighbouring cells*/
	int j;
	for (j = 0; j < 26; j++) {

		current_cell = neighbours[cell_of][j];
		if (current_cell == -1) break;
		i = head_of[ current_cell ];
		while (i != -1) {
			energy += (*potential)(GEO_distance(x,y,z,x_list[i],y_list[i],z_list[i],
							periodic, paramsG),
					paramsP);
			i = ll[i];	
		}
		

	}
	
	return energy;	

}

/*FUNCTION: energy_total is used to calculate the  energy of
 * the entire system in units of epsilon/kt
 *
 * Output: type double: The energy
 * Parameters:
 *	Mc_state state
 *	Mc_cell cell: the cell list
 *	double(*potential) a function which returns the potential felt by two particles
 *	separated by a set distance (the first argument).
 *	paramsP is the parameter list for the user-defined potential function
 */

double EGY_energy_total(Mc_state *state, Mc_cell *cell, int periodic, double *paramsG,
			double (*potential)(double,double*), double paramsP[]) {

	int i;
	double energy = 0;
	for (i = 0; i < state->n_particles; i++) {
		energy += EGY_energy_calc(state,cell,state->x[i],state->y[i],state->z[i],
				i,cell->of_particle[i], periodic, paramsG,
				potential,paramsP);
	}
	return energy/paramsP[0];
}

/*Fruction: energy_list returns a list containing the energy of each particle in the system
 *
 * Output: the list of energies
 * Parameters:
 *	Mc_state state
 *	Mc_cell cell: the cell list
 **/
double * EGY_energy_list (Mc_state *state, Mc_cell *cell, int periodic, double *paramsG,
			double (*potential)(double,double*), double paramsP[]) {

	double *energy = malloc(sizeof(double)*state->n_particles);
	int i;
	for (i = 0; i < state->n_particles; i++) {
		energy[i] = EGY_energy_calc(state,cell,state->x[i],state->y[i],state->z[i],
				i,cell->of_particle[i], periodic, paramsG,
				potential,paramsP)/paramsP[0];
	}
	return energy;

}

