/*A module for generating trial moves on different surfaces for monte carlo simulation.
 * The distribution of moves is garunteed to be uniform
 * the module prefix is GEO_
 */

#include"mc_math_h.h"
#include"std_h.h"
#include"twister_h.h"
#include"geometry_h.h"

/*Function GEO_mv_sphere: generates well sampled purterbations of a particle confined to a sphere
 * params x,y,z: the coordinates of the particle (to be updated)
 * input: *radius pointer to the radius of the sphere 
 * input: delta_r, the amount by which the position can change
 */

void GEO_mv_sphere (double *x, double *y, double *z, double *radius, double delta_r) {

	double ransq = 2.0;
	double ran1, ran2;

	while (ransq > 1.0) {
		ran1 = 1.0 - 2.0*randomF();
		ran2 = 1.0 - 2.0*randomF();
		ransq = ran1*ran1 + ran2*ran2;
	}

	double ranh = 2.0*sqrt(1.0 - ransq);
	
	*x = *x + ran1*ranh*delta_r;
	*y = *y + ran2*ranh*delta_r;
	*z = *z + delta_r*(1.0 - 2.0*ransq);

	ransq = sqrt( (*x)*(*x) + (*y)*(*y) + (*z)*(*z) );
	*x = (*x) * (*radius)/(ransq);
	*y = (*y) * (*radius)/(ransq);
	*z = (*z) * (*radius)/(ransq);

}

/*gets a point at random, uniformly distributed on the sphere
 *
 * parms (x,y,x) the coords of the particle
 * input: radius: the radius of the particle
 */

void GEO_random_sphere (double *x, double *y, double *z, double *radius) {

	double ransq = 2.0;
	double ran1, ran2;
	while (ransq > 1.0) {
		ran1 = 1.0 - 2.0*randomF();
		ran2 = 1.0 - 2.0*randomF();
		ransq = ran1*ran1 + ran2*ran2;
	}
	double ranh = 2.0*sqrt(1.0 - ransq);
	*x = ran1*ranh* (*radius);
	*y = ran2*ranh* (*radius);
	*z = (1.0 - 2.0*ransq)* (*radius);
}

/*get the area of a sphere of a given radius*/
double GEO_area_sphere(double *radius) {
	return 4*M_PI*(*radius)*(*radius);
}

/*FUNCTION GEO_mv_plane: generates well sampled moves of a particle confined to the x-y plane
 *
 * params x,y,z: the coordinates of the particle (to be updated)
 * params: the x,y lengths of the box (box is centred on the origin
 * input: delta: the amount theat the particle can move in a give ndirection
 */

void GEO_mv_plane(double *x, double *y, double *z, double *params, double delta) {

	*x += (2*randomF()-1)*delta;
	*y += (2*randomF()-1)*delta;

	if (*x > params[0]/2.0) {
		*x -= params[0];
	} else if (*x < -1*params[0]/2.0) {
		*x += params[0];
	}

	if (*y > params[1]/2.0) {
		*y -= params[1];
	} else if (*y < -1*params[1]/2.0) {
		*y += params[1];
	}


}

/*FUNCTION GEO_radom plane: randomly places a particle in a box
 *
 * Params x,y,z: the output location of the particel
 * params: the x,y lengths of the box (box is centred on the origin
 */

void GEO_random_plane(double *x, double *y, double *z, double *params) {
	*x = (randomF()-0.5)*params[0];
	*y = (randomF()-0.5)*params[1];
	*z = 0;
}

/*get the size of a plane with a given edge length*/
double GEO_area_plane(double *params) {
	return params[0]*params[1];
}


/*FUNCTION GEO_mv_torus: generates well sampled moves of a particle confined to a torus
 *
 * params x,y,z: the coordinates of the particle (to be updated)
 * params: a,c to describe the torus
 * input: delta: the amount theat the particle can move in either angle
 */
void GEO_mv_torus(double *x, double *y, double *z, double *params, double delta) {
	double u = atan2(*y,*x);
	double v = asin((*z)/params[0]);
	if ((*x)*(*x) + (*y)*(*y) < params[1]*params[1]) {
		if ((*z) > 0) {
			v = M_PI - v;
		} else {
			v = - M_PI - v;
		}
	}

	double u_new = u + (2*randomF()-1)*delta;
	double v_new = v + (2*randomF()-1)*delta;

	if (randomF() < (params[1] + params[0]*cos(v_new))/(params[1] + params[0]*cos(v))) {
		*x = (params[1]+params[0]*cos(v_new))*cos(u_new);
		*y = (params[1]+params[0]*cos(v_new))*sin(u_new);
		*z = params[0]*sin(v_new);
	}
}

/*FUNCTION GEO_radom torus: randomly places a particle on a torus
 *
 * Params x,y,z: the output location of the particel
 * params: a,c to describe the torus
 */
void GEO_random_torus(double *x, double *y, double *z, double *params) {

	double u = randomF()*2*M_PI;
	double v = randomF()*2*M_PI;

	do {
		v = randomF()*2*M_PI;
	} while (randomF() > (1+(params[0]/params[1])*cos(v))/(1+(params[0]/params[1])));

	*x = (params[1]+params[0]*cos(v))*cos(u);
	*y = (params[1]+params[0]*cos(v))*sin(u);
	*z = params[0]*sin(v);
}

/*get the size of a torus with any parameters*/
double GEO_area_torus(double *params) {
	return 4*M_PI*M_PI*(params[0])*(params[1]);
}

/*Function: GEO_distance get s the real distance between two points in space
 *
 *output: the distance
 *input: the coordinates in cartesian space of particles 1 and 2
 */

double GEO_distance (double x1, double y1, double z1, double x2, double y2, double z2,
			int periodic, double *params) {
	if (!periodic) {
		return sqrt( (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2) );
	} else {
		double dx = x1-x2;
		if (dx < -0.5*params[0]) {
			dx += params[0];
		} else if (dx > 0.5*params[0]) {
			dx -+ params[0];
		}
		double dy = y1-y2;
		if (dy < -0.5*params[1]) {
			dy += params[1];
		} else if (dy > 0.5*params[1]) {
			dy -+ params[1];
		}
		double dz = z1-z2;
		if (dz < -0.5*params[0]) {
			dz += params[0];
		} else if (dz > 0.5*params[0]) {
			dz -+ params[0];
		}
		return sqrt(dx*dx + dy*dy + dz*dz);
	}

}


