/* A module that contains functions for performig monte carlo moves in two dimensions,
 * utilising a cell list which exists in three dimensions around the surface.
 * 
 *
 * The prefix is mc_
 */

#include"mc_state_h.h"
#include"cell_list_h.h"
#include"mc_math_h.h"
#include"std_h.h"
#include"energy_h.h"
#include"geometry_h.h"
#include"twister_h.h"
#include"monte_carlo_h.h"

/*Function: Initialise. This function places particles in a system State. 
 *      If READFILE is no, then N particles (N read from Input struct) will be 
 *	placed into the system at random.
 *
 *	If READFILE is yes, then the state will be loaded from files "final.csv"
 *	into the state struct. The values in range are assumed to be correct and not checked.
 *
 *
 * input int readFile: 1 if file is to be read, 0 otherwise
 * int n_particles: The number of particles required
 * random: gets a random, well sampled position on the surface of interest
 */

void mc_initialise (Mc_state *state, int readFile, int restart, int n_particles, double *params, 
			char run_id[], void (*random)(double *, double *, double *,double *),
			double *deltaR, double *deltaC, double *deltaS) {

	FILE *fp;
	int i;
	char command[50];
	if (readFile) {
		sprintf(command,"../state%s.csv",run_id);
		fp = fopen(command,"r");
		fscanf(fp,"%d %*[\n]",&n_particles);
		fscanf(fp,"%lf %*[\n]",deltaR);
		fscanf(fp,"%lf %*[\n]",deltaC);
		fscanf(fp,"%lf %*[\n]",deltaS);

		state->n_particles = n_particles;
		state->x = malloc(sizeof(double) * n_particles);
		state->y = malloc(sizeof(double) * n_particles);
		state->z = malloc(sizeof(double) * n_particles);

		for (i = 0; i < n_particles; i++) {
			fscanf(fp,"%lf %*c %lf %*c %lf %*[\n]", (state->x)+i, (state->y)+i,
										(state->z)+i);
		}
		fclose(fp);
	} else if (restart) {
		sprintf(command,"state%s.csv",run_id);
		fp = fopen(command,"r");
		fscanf(fp,"%d %*[\n]",&n_particles);
		fscanf(fp,"%lf %*[\n]",deltaR);
		fscanf(fp,"%lf %*[\n]",deltaC);
		fscanf(fp,"%lf %*[\n]",deltaS);
		state->n_particles = n_particles;
		state->x = malloc(sizeof(double) * n_particles);
		state->y = malloc(sizeof(double) * n_particles);
		state->z = malloc(sizeof(double) * n_particles);

		for (i = 0; i < n_particles; i++) {
			fscanf(fp,"%lf %*c %lf %*c %lf %*[\n]", (state->x)+i, (state->y)+i,
										(state->z)+i);
		}
		fclose(fp);
	} else {
		state->n_particles = n_particles;
		state->x = malloc(sizeof(double) * n_particles);
		state->y = malloc(sizeof(double) * n_particles);
		state->z = malloc(sizeof(double) * n_particles);
		for (i = 0; i<n_particles; i++) {
			random( (state->x)+i, (state->y)+i, (state->z)+i, params);
		}

	}



}

/*Function: particle_move. Performs a full single particle move on one particle in the state
 *
 * Output: 1 if the move is accepted, 0 otherwise
 * Param: State: the current state mc_state struct for the system (to be updated)
 * Param: cell: the current (up to date) cell list for the system (to be updated)
 * input: 
 *	double(*potential) a function which returns the potential felt by two particles
 *	separated by a set distance (the first argument).
 *	(*move) a function that suggest random moves for the particle with coords that are the 
 *	first three arguments
 *	paramsP is the parameter list for the user-defined potential function
 *	paramsD is some paramters to pss to the move function for the move function
 *		as is delta_r
 **/

int mc_particle_move (Mc_state *state, Mc_cell *cell, int periodic, double delta_r,
			double (*potential)(double,double*),
			void (*move) (double*,double*,double*,double*,double),
			double paramsP[], double paramsD[]) {


	if (state->n_particles ==0) return 0;

	//select a particle to move
	int mover = randomMT() % state->n_particles;

	//get ol state
	double old_x = state->x[mover];
	double old_y = state->y[mover];
	double old_z = state->z[mover];

	double old_energy  = EGY_energy_calc(state,cell,old_x,old_y,old_z,mover,
					cell->of_particle[mover], periodic, paramsD, 
					potential, paramsP);

	move( (state->x)+mover, (state->y)+mover, (state->z)+mover, paramsD, delta_r);
	CLL_update_particle(cell, state,mover);
	//get new energy
	double energy = EGY_energy_calc(state,cell,state->x[mover], state->y[mover], state->z[mover],
						mover, cell->of_particle[mover],periodic,paramsD,
						potential,paramsP);
	
	//accept of reject
	if (randomF() < exp (-1*(energy - old_energy)) ) {
		return 1;
	} else {
		state->x[mover] = old_x;
		state->y[mover] = old_y;
		state->z[mover] = old_z;
		CLL_update_particle(cell,state,mover);
		return 0;
	}	
}

/*
 * Function: particle_add. Performs a grand cannonical particle addition step
 * Output: 1 if the move is accepted, 0 otherwise
 * Param: State: the current state mc_state struct for the system (to be updated)
 * Param: cell: the current (up to date) cell list for the system (to be updated)
 *
 *
 * input: 
 *	double(*potential) a function which returns the potential felt by two particles
 *	separated by a set distance (the first argument).
 * 	random: gets a random, well sampled position on the surface of interest
 *	(*move) a function that suggest random moves for the particle with coords that are the 
 *	paramsP is the parameter list for the user-defined potential function
 *	paramsD is the parameter list for the user-defined geometry function
 *	activity is the activity of the gc ensemble
 */


int mc_particle_add (Mc_state *state, Mc_cell *cell, int periodic,
			double (*potential)(double,double*),
			void (*random)(double *, double *, double *,double *),
			double paramsP[], double paramsD[], double activity) {

	double x_temp, y_temp, z_temp;
	double *temptr;	
	double *temptr2;
	double *temptr3;

	random(&x_temp,&y_temp,&z_temp,paramsD);
	long temp_cell = CLL_get_index(x_temp,y_temp,z_temp,cell);
	
	//get energy
	double energy = EGY_energy_calc(state,cell,x_temp,y_temp,z_temp,state->n_particles + 1,
						temp_cell, periodic, paramsD, potential,paramsP);

	if (randomF() < (activity/((float)(state->n_particles)+1.0))*exp(-1*energy)) {
		/*accept*/
		
		state->n_particles++;
		temptr = realloc(state->x, sizeof(double)*state->n_particles);
		if (temptr != NULL) {
			state->x = temptr;
		} else {
			die ("realloc failed in add");
		}		
		
		temptr2 = realloc(state->y,sizeof(double)*state->n_particles);
		if (temptr2 != NULL) {
			state->y = temptr2;
		} else {
			die ("realloc failed in add");
		}

		temptr3 = realloc(state->z,sizeof(double)*state->n_particles);
		if (temptr3 != NULL) {
			state->z = temptr3;
		} else {
			die ("realloc failed in add");
		}

		state->x[state->n_particles -1] = x_temp;
		state->y[state->n_particles -1] = y_temp;
		state->z[state->n_particles -1] = z_temp;

		CLL_add_particle(cell,state,state->n_particles -1);
		return 1;
	} else {
		return 0;
	}
	
	return 0;
	
}


/*
 * Function: particle_remove. Performs a grand cannonical particle removal step
 * Output: 1 if the move is accepted, 0 otherwise
 * Param: State: the current state mc_state struct for the system (to be updated)
 * Param: cell: the current (up to date) cell list for the system (to be updated)
 *
 *
 * input: 
 *	double(*potential) a function which returns the potential felt by two particles
 *	separated by a set distance (the first argument).
 *	(*move) a function that suggest random moves for the particle with coords that are the 
 *	paramsP is the parameter list for the user-defined potential function
 *	paramsD is the parameter list for the user-defined geometry function
 *	activity is the activity of the gc ensemble
 */

int mc_particle_remove (Mc_state *state, Mc_cell *cell, int periodic,
			double (*potential)(double,double*),
			double paramsP[], double paramsD[], double activity) {

	if (state->n_particles == 0) return 0;
	/*select a particle to remove*/
	int mover = randomMT() % state->n_particles;
	
	/*get energy*/
	double energy = EGY_energy_calc(state,cell,state->x[mover],state->y[mover],state->z[mover],
						mover,cell->of_particle[mover], periodic,
						paramsD,potential,paramsP);

	if (randomF() < (((float) state->n_particles)/activity)*exp(energy)) {
	
		double *temptr;	
		double *temptr2;
		double *temptr3;

		//accept
		CLL_remove_particle(cell,state,mover);

		//remove from lists
		
		int i;
		for (i = mover; i< (state->n_particles)-1;i++) {
			state->x[i] = state->x[i+1];
			state->y[i] = state->y[i+1];
			state->z[i] = state->z[i+1];
		}

		state->n_particles--;
		temptr = realloc(state->x, sizeof(double)*state->n_particles);
		if (temptr || state->n_particles == 0) {
			state->x = temptr;
		} else {
			die ("realloc failed in remove");
		}		
		
		temptr2 = realloc(state->y,sizeof(double)*state->n_particles);
		if (temptr2 || state->n_particles == 0) {
			state->y = temptr2;
		} else {
			die ("realloc failed in remove");
		}

		temptr3 = realloc(state->z,sizeof(double)*state->n_particles);
		if (temptr3 || state->n_particles == 0) {
			state->z = temptr3;
		} else{
			die ("realloc fialed in remove");
		}

		return 1;

	} else {
		return 0;
	}
	
	return 0;

}

/*
 * Function mc_Record: Spits out a frame and attaches it to the end of history.csv
 * Input: State: the current state of the system
 */

void mc_record(Mc_state *state) {
	FILE *fp;
	int i;

	fp = fopen ("history.csv","a");
	fprintf(fp,"%ld,",state->n_particles);
	for (i = 0; i < state->n_particles; i++) {
		fprintf(fp,"%f,%f,%f,",state->x[i],state->y[i],state->z[i]);
	}
	fprintf(fp,"\n");
	fclose(fp);
}


/*
 *Function mc_record_neighbours: spits out the value of some order parameter function for each
 * particle in a way that can be used by visulaisation programs
 */

void mc_record_order( double *orders, int n, char *name ) {

	FILE *fp;
	
	char command[50];	
	sprintf(command,"%s_history.csv",name);

	fp = fopen(command,"a");
	fprintf(fp,"%d,", n);
	int i;
	for (i= 0; i < n; i++) {
		fprintf(fp,"%lf,", orders[i]);
	}
	fprintf(fp,"\n");
	fclose(fp);
}

/*Function: MC_save: saves the current state of the system: can be used for restarts*/
void mc_save(Mc_state *state, char run_id[], double delta_r, double deltaC, double deltaS) {
	FILE  *fp;
	int i;
	char command[50];
	sprintf(command,"state%s.csv",run_id);
	fp = fopen(command,"w");
	fprintf(fp,"%ld\n",state->n_particles);
	fprintf(fp,"%lf\n",delta_r);
	fprintf(fp,"%lf\n",deltaC);
	fprintf(fp,"%lf\n",deltaS);
	for (i = 0; i < state->n_particles; i++) {
		fprintf(fp,"%lf,%lf,%lf\n",state->x[i],state->y[i],state->z[i]);
	}
	fclose(fp);
}





