/*This is a module for handling umbrella sampling in order to study nucleation
 * and on a surface
 *
 *the module prefix is _UMB
 */
#include"mc_state_h.h"
#include"cell_list_h.h"
#include"geometry_h.h"
#include"std_h.h"
#include"mc_math_h.h"
#include"twister_h.h"
#include"replica_exchange_h.h"
#include"mpi_h.h"
#include"umbrellaRE_h.h"

/* FUNCTION: constructor: makes a new histogram big enough to contain all posible nuclear sizes
 * 
 * Input: histogram: the histogram to be initialised
 * Input: Volume: the volume of the system in units of particle volume
 * Input: *name: the name of the histogram
 */
void UMB_new_histogram(Mc_umbrella *histogram, double volume, char *name, int restart) {
	int saturation = (int) (volume/(0.25*M_PI));
	histogram->max_size = (int) (1.5*saturation);
	histogram->isScaled = 0;
	histogram->freq = malloc(sizeof(long)*histogram->max_size);
	histogram->scaled = malloc(sizeof(double)*histogram->max_size);
strcpy(histogram->name,name);
	int i;

	if (restart) {

		char command[50];
		sprintf(command,"%s_frequencies.csv",histogram->name);
		FILE *fp;
		fp = fopen(command,"r");
		for (i = 0; i < histogram->max_size; i++) {
			fscanf(fp,"%*d %*c %ld %*[\n]",(histogram->freq)+i);
			histogram->scaled[i] = 0.0;
		}
		fclose(fp);

	} else {
		for(i = 0; i < histogram->max_size; i++) {
			histogram->freq[i] = 0;
			histogram->scaled[i] = 0.0;
		}
	}
}

/*Function add value: add a new nucleus to the histogram
 *
 * Input: histogram: the histogram to be initialised
 * Input: value: the value to be added
 *
 */
void UMB_add_value(Mc_umbrella *histogram, int value) {
	histogram->isScaled = 0;

	if (value < histogram->max_size) {
		histogram->freq[value]++;
	}
}

/*FUNCTION add array: adds one to the bind of each index in array*/
void UMB_add_array(Mc_umbrella *histogram, int *array, int length) {
	histogram->isScaled = 0;
	int i;
	for (i = 0 ; i < length; i++) {
		if (array[i] < histogram->max_size) {
			histogram->freq[array[i]] ++;
		}
	}

}

void UMB_add_array_scale(Mc_umbrella *histogram, int *array, int length, double lambda,int n_target,
			int nucleus) {
	histogram->isScaled = 1;
	int i;
	//make histogram of array//
	double * temp = malloc (sizeof(double) * histogram->max_size);
	for (i = 0; i < histogram->max_size;i++) {
		temp[i] = 0.0;
	}
	for (i = 0; i < length; i++) {
		if (array[i] < histogram->max_size) {
			temp[array[i]] += 1.0;
		}
	}
	//scale temp and add
	for (i = 0; i < histogram->max_size; i++) {
		histogram->scaled[i] += temp[i]/(exp(-0.5*lambda*(nucleus-n_target)*(nucleus-n_target)));
	}
	free(temp);
}

/* Function: scale:  scales the histogram acording to the rules of umbrella sampling
 *
 * Input: Mc_umbrella : the histogram to be scaled
 * Input: lambda: the spring constant of the umbrella potential
 *input : n_target: The target nuclear size
 */

void UMB_scale(Mc_umbrella *histogram, double lambda, int n_target) {
	histogram->isScaled = 1;
	int i;
	for (i = 0; i < histogram->max_size; i++) {
		histogram->scaled[i] = (double) histogram->freq[i] / 
					        (exp(-0.5*lambda*(i-n_target)*(i-n_target)) );
	}

}


void UMB_sort_histogram_scale(Mc_umbrella *final_histogram, Mc_umbrella *histograms, int size) {
	int i;
	int n_bins = histograms[0].max_size;
	long *temp_freq;
	long *new_freq;
	new_freq = malloc(sizeof(long)*n_bins);
	double *temp_scale;
	double *new_scale;
	new_scale = malloc(sizeof(double)*n_bins);

	for (i=0; i < size; i++) {
		temp_freq = histograms[i].freq;
		MPI_Reduce (temp_freq, new_freq, n_bins, MPI_LONG, MPI_SUM, i ,MPI_COMM_WORLD);
		temp_scale = histograms[i].scaled;
		MPI_Reduce (temp_scale,new_scale, n_bins, MPI_DOUBLE,MPI_SUM,i,MPI_COMM_WORLD);
	}
	free(final_histogram->freq);
	free(final_histogram->scaled);
	final_histogram->freq = new_freq;
	final_histogram->scaled = new_scale;
}

void UMB_sort_histogram (Mc_umbrella *final_histogram, Mc_umbrella *histograms, int size) {

	int i;
	int n_bins = histograms[0].max_size;
	long *temp_freq;
	long *new_freq;
	new_freq = malloc (sizeof(long)*n_bins);
	for ( i =0; i < size; i++) {
		temp_freq = histograms[i].freq;
		MPI_Reduce (temp_freq, new_freq, n_bins, MPI_LONG, MPI_SUM, i ,MPI_COMM_WORLD);
	}
	free(final_histogram->freq);
	final_histogram->freq = new_freq;

}


/* Function: print: prints out both the raw feqecncy and scaled histogram data
 *
 * Input histogramL teh histgoram to be printed
 **/

void UMB_print(Mc_umbrella *histogram) {

	int i;
	FILE *fp;
	char command[50];

	sprintf(command,"%s_frequencies.csv",histogram->name);
	fp =fopen(command,"w");
	for (i = 0; i < histogram->max_size; i++) {
		fprintf(fp,"%d,%ld\n",i,histogram->freq[i]);
	}

	fclose(fp);	

	sprintf(command,"%s_scaled.csv",histogram->name);
	fp = fopen(command,"w");

	if (histogram->isScaled == 0) {
		fprintf(fp,"Scaled histogram is not up to date with frequencies\n");
	}

	for (i = 0 ; i < histogram->max_size; i++) {
		fprintf(fp,"%d,%lf\n",i,histogram->scaled[i]);
	}
	fclose(fp);

}

/*Destructor*/
void UMB_destroy(Mc_umbrella *histogram) {
	free(histogram->freq);
	free(histogram->scaled);

}
/*FUNCTION get  nuclei: This function returns the size of the largest nucleus in the
 * system. A nucleus is defined as a cluster of particles that have an order paramter of at
 * least *o_cutoff* and are within *d_cutoff* of each other
 *
 * OUtput: a list containg all the nuclear sizes (of size n_partticles with trailing zeroes)
 * Input: state: the state of the system
 * input: o_cutoff: the order parameter cutoff
 * input: d_cutoff: the neighbour distence cutoff
 * input order: a list of the order parameter of each particle
 */ 

int * UMB_get_nucleus (Mc_state *state, double o_cutoff, double d_cutoff, double *order, 
			int periodic, double *paramsG) {
	/*special case*/
	if (state->n_particles < 2) {
		int *one_ptr = malloc (sizeof(int));
		one_ptr[0] = 0;
		return one_ptr;
	}
	/*make clusters*/
	int *hoc = malloc(sizeof(int)*state->n_particles);
	int *ll =  malloc(sizeof(int)*state->n_particles);
	int i,j,n;

	/*init*/
	for (i = 0; i < state->n_particles;i++) {
		hoc[i] = i;
		ll[i]  =-1;
	}

	/*loop over pairs*/
	for(i = 0; i < state->n_particles -1; i++) {
		for(j = i+1; j < state->n_particles; j++) {
			if (order[i] > o_cutoff && order[j] > o_cutoff &&
			    GEO_distance(state->x[i],state->y[i],state->z[i],
					 state->x[j],state->y[j],state->z[j],
					 periodic, paramsG) < d_cutoff) {
				
				/*head of list i to tail of list j*/
				if (hoc[i] != hoc[j]) {
					n = i;
					while (ll[n] != -1) {
						n = ll[n];
					}
					ll[n] = hoc[j];
					n = hoc[j];
					while (n != -1) {
						hoc[n] = hoc[i];
						n = ll[n];
					}
				}
			}
		}
	}

	/*get number of clusters*/
	int n_clusters = 0;
	/*make visited list*/
	int *visited = malloc(sizeof(int)*state->n_particles);
	int *clusters = malloc(sizeof(int)*state->n_particles);
	for (i = 0; i < state->n_particles; i++) {
		visited[i] = 0;
		clusters[i] = 0;
	}

	for ( i = 0; i< state->n_particles; i++) {
		if (visited[i]) continue;
		n = hoc[i];
		n_clusters ++;
		while (n != -1) {
			visited[n] = 1;
			clusters[n_clusters-1]++;
			n = ll[n];	
		}	
	}

	free(visited);
	free(hoc);
	free(ll);
	return clusters;
}

/*FUCNTION: record largest nucleus: reutns a list which contins whether or not each
 * particle is in the largest nucleus. 1 if it is and 0 id it is not
 *
 * OUtput: a list containg all the nuclear sizes (of size n_partticles with trailing zeroes)
 * Input: state: the state of the system
 * input: o_cutoff: the order parameter cutoff
 * input: d_cutoff: the neighbour distence cutoff
 * input order: a list of the order parameter of each particle
 */
double * UMB_record_largest(Mc_state *state, double o_cutoff, double d_cutoff, double *order,
				int periodic, double *paramsG) {

	/*special case*/
	if (state->n_particles < 2) {
		double *one_ptr = malloc (sizeof(double));
		one_ptr[0] = 0.0;
		return one_ptr;
	}

	/*make clusters*/
	int *hoc = malloc(sizeof(int)*state->n_particles);
	int *ll =  malloc(sizeof(int)*state->n_particles);
	int i,j,n;

	/*init*/
	for (i = 0; i < state->n_particles;i++) {
		hoc[i] = i;
		ll[i]  =-1;
	}

	/*loop over pairs*/
	for(i = 0; i < state->n_particles -1; i++) {
		for(j = i+1; j < state->n_particles; j++) {
			if (order[i] > o_cutoff && order[j] > o_cutoff &&
			    GEO_distance(state->x[i],state->y[i],state->z[i],
					 state->x[j],state->y[j],state->z[j],
					 periodic ,paramsG) < d_cutoff) {
				
				/*head of list i to tail of list j*/
				if (hoc[i] != hoc[j]) {
					n = i;
					while (ll[n] != -1) {
						n = ll[n];
					}
					ll[n] = hoc[j];
					n = hoc[j];
					while (n != -1) {
						hoc[n] = hoc[i];
						n = ll[n];
					}
				}
			}
		}
	}

	/*make visited list*/
	int *visited = malloc(sizeof(int)*state->n_particles);
	double *in_largest = malloc(sizeof(double)*state->n_particles);
	int head_of_largest = -1;
	int size_of_largest = 0.0;
	int size;
	for (i = 0; i < state->n_particles; i++) {
		visited[i] = 0;
		in_largest[i] = 0;
	}

	for ( i = 0; i< state->n_particles; i++) {
		if (visited[i]) continue;
		n = hoc[i];
		size = 0;
		while (n != -1) {
			visited[n] = 1;
			size++;
			n = ll[n];	
		}
		if (size > size_of_largest) {
			size_of_largest = size;
			head_of_largest = hoc[i];
		}	
	}

	n = head_of_largest;
	while(	n!= -1) {
		in_largest[n] = 1.0;
		n = ll[n];
	}

	free(visited);
	free(hoc);
	free(ll);
	return in_largest;

}

/*FUNCTION umbreall_move_largest: This function simply checks whether the last batch of
 *mc moves is accepted, or whether it is reverted. It is based on the size of the largest cluster
 *
 *Output: 1 if the move is aacpected, 0 otherwise
 *Param: *mc_state state: the current state of the system
 *Param: *mc_state old_state: the old state of the system
 *Output: u_rec, the final nulcear size for this system
 *param: u_old: the previous size of the largest nucleus in the system
 * input: lambda: the weighting of the umbrealla sampling bias energy
 * input n_target: the target nuclear size
 * input: o_cutoff: the order parameter cutoff
 * input :d_cutoff: the neighbour distence cutoff
 * intput: double * order: a list of the order param of each particle
 */

int UMB_move_largest (Mc_state *state, Mc_state *old_state, Mc_cell *cell, Mc_cell *old_cell, 
			int *u_rec, int *u_old, double lambda, int n_target,
			double o_cutoff, double d_cutoff, double *order,
			int periodic, double *paramsG) {
	
	int * nuclei = UMB_get_nucleus (state, o_cutoff, d_cutoff,order,periodic,paramsG);
	*u_rec = array_max_i(nuclei,state->n_particles);

	/*get energies*/
	//printf("%f %d\n",lambda,n_target);
	double energy_old = 0.5*lambda*( (*u_old)-n_target) * ( (*u_old)-n_target);
	double energy_new = 0.5*lambda*( (*u_rec)-n_target) * ( (*u_rec)-n_target);
	
	if (randomF() < exp (-1 * (energy_new - energy_old)) ) {
		*u_old = *u_rec;
		CLL_cell_deepCopy ( old_cell, cell, state->n_particles, old_state->n_particles );
		MCS_state_deepCopy ( old_state, state);
		free(nuclei);
		return 1;
	} else {
		*u_rec = *u_old;
		CLL_cell_deepCopy (cell, old_cell, old_state->n_particles, state->n_particles);
		MCS_state_deepCopy (state, old_state);
		free(nuclei);
		return 0;
	}
}

/*FUNCTION UMB_move_max: This fuction checks whether the latest batch of umbrella moves
 * is accepted or rejected. All moves with nucleus > n_max get rejected, otherwise accpeted.
 *
 *Output: 1 if the move is accpected, zero otherwise
 *Param: *mc_state state: the current state of the system
 *Param: *mc_state old_state: the old state of the system
 *Output: u_rec, the final nulcear size for this system
 *param: u_old: the previous size of the largest nucleus in the system
 * input n_max: the largest allowed nucleus
 * input: o_cutoff: the order parameter cutoff
 * input :d_cutoff: the neighbour distence cutoff
 * intput: double * order: a list of the order param of each particle
 */

int UMB_move_max (Mc_state *state, Mc_state *old_state, Mc_cell *cell, Mc_cell *old_cell,
		  int *u_rec, int *u_old,int n_max, double o_cutoff, double d_cutoff, double *order,
		  int periodic, double *paramsG) {

	int * nuclei = UMB_get_nucleus (state, o_cutoff, d_cutoff,order,periodic,paramsG);
	*u_rec = array_max_i(nuclei,state->n_particles);

	if (*u_rec <= n_max) {
		*u_old = *u_rec;
		CLL_cell_deepCopy ( old_cell, cell, state->n_particles, old_state->n_particles );
		MCS_state_deepCopy ( old_state, state);
		free(nuclei);
		return 1;
	} else {
		*u_rec = *u_old;
		CLL_cell_deepCopy (cell, old_cell, old_state->n_particles, state->n_particles);
		MCS_state_deepCopy (state, old_state);
		free(nuclei);
		return 0;
	}
}










