#ifndef UMBRELLA_H
#define UMBRELLA_H

/*This is a module for handling umbrella sampling in order to study nucleation
 * and on a surface
 *
 *the module prefix is _UMB
 */

/*Struct: for storing the size of the largest nucleus as a histogram*/

typedef struct umbrstrc {

	int max_size;
	long *freq;
	double *scaled;
	int isScaled;
	char name[50];
	
} Mc_umbrella;

/* FUNCTION: constructor: makes a new histogram big enough to contain all posible nuclear sizes
 * 
 * Input: histogram: the histogram to be initialised
 * Input: Volume: the volume of the system in units of particle volume
 * Input: *name: the name of the histogram
 */
void UMB_new_histogram(Mc_umbrella *histogram, double volume, char *name, int restart);

/*Function add value: add a new nucleus to the histogram
 *
 * Input: histogram: the histogram to be initialised
 * Input: value: the value to be added
 **/
void UMB_add_value(Mc_umbrella *histogram, int value);

/*FUNCTION add array: adds one to the bind of each index in array*/
void UMB_add_array(Mc_umbrella *histogram, int *array, int length) ;

void UMB_add_array_scale(Mc_umbrella *histogram, int *array, int length, double lambda,int n_target,
			int nucleus);

/* Function: scale:  scales the histogram acording to the rules of umbrella sampling
 *
 * Input: Mc_umbrella : the histogram to be scaled
 * Input: lambda: the spring constant of the umbrella potential
 *input : n_target: The target nuclear size
 */

void UMB_scale(Mc_umbrella *histogram, double lambda, int n_target);

/* Function: print: prints out both the raw feqecncy and scaled histogram data
 *
 * Input histogramL teh histgoram to be printed
 **/

void UMB_print(Mc_umbrella *histogram);

/*Destructor*/
void UMB_destroy(Mc_umbrella *histogram);

/*FUNCTION get  nuclei: This function returns the size of the largest nucleus in the
 * system. A nucleus is defined as a cluster of particles that have an order paramter of at
 * least *o_cutoff* and are within *d_cutoff* of each other
 *
 * OUtput: a list containg all the nuclear sizes (of size n_partticles with trailing zeroes)
 * Input: state: the state of the system
 * input: o_cutoff: the order parameter cutoff
 * input: d_cutoff: the neighbour distence cutoff
 * input order: a list of the order parameter of each particle
 */ 

int * UMB_get_nucleus (Mc_state *state, double o_cutoff, double d_cutoff, double *order, 
			int periodic, double *paramsG);

/*FUCNTION: record largest nucleus: reutns a list which contins whether or not each
 * particle is in the largest nucleus. 1 if it is and 0 id it is not
 *
 * OUtput: a list containg all the nuclear sizes (of size n_partticles with trailing zeroes)
 * Input: state: the state of the system
 * input: o_cutoff: the order parameter cutoff
 * input: d_cutoff: the neighbour distence cutoff
 * input order: a list of the order parameter of each particle
 */
double * UMB_record_largest(Mc_state *state, double o_cutoff, double d_cutoff, double *order,
				int periodic, double *paramsG);

/*FUNCTION umbreall_move_largest: This function simply checks whether the last batch of
 *mc moves is accepted, or whether it is reverted. It is based on the size of the largest cluster
 *
 *Output: 1 if the move is aacpected, 0 otherwise
 *Param: *mc_state state: the current state of the system
 *Param: *mc_state old_state: the old state of the system
 *Output: u_rec, the final nulcear size for this system
 *param: u_old: the previous size of the largest nucleus in the system
 * input: lambda: the weighting of the umbrealla sampling bias energy
 * input n_target: the target nuclear size
 * input: o_cutoff: the order parameter cutoff
 * input :d_cutoff: the neighbour distence cutoff
 * intput: double * order: a list of the order param of each particle
 */

int UMB_move_largest (Mc_state *state, Mc_state *old_state, Mc_cell *cell, Mc_cell *old_cell, 
			int *u_rec, int *u_old, double lambda, int n_target,
			double o_cutoff, double d_cutoff, double *order,
			int periodic, double *paramsG) ;

/*FUNCTION UMB_move_max: This fuction checks whether the latest batch of umbrella moves
 * is accepted or rejected. All moves with nucleus > n_max get rejected, otherwise accpeted.
 *
 *Output: 1 if the move is accpected, zero otherwise
 *Param: *mc_state state: the current state of the system
 *Param: *mc_state old_state: the old state of the system
 *Output: u_rec, the final nulcear size for this system
 *param: u_old: the previous size of the largest nucleus in the system
 * input n_max: the largest allowed nucleus
 * input: o_cutoff: the order parameter cutoff
 * input :d_cutoff: the neighbour distence cutoff
 * intput: double * order: a list of the order param of each particle
 */

int UMB_move_max (Mc_state *state, Mc_state *old_state, Mc_cell *cell, Mc_cell *old_cell,
		  int *u_rec, int *u_old,int n_max, double o_cutoff, double d_cutoff, double *order,
		  int periodic, double *paramsG);

#endif
