/*................................................................................

hist.c

Version 4

Multiple histogram for 2D sticky sphere histograms of the number of particles and
number of contacts.  Input is a series of histograms (as produced by sticky.c
version 15), which must be measured at different values of tau, but the same
activity.  The filenames for the histograms and the values of tau at which they
were measured are supplied in an external file.  The name of this list file is
supplied as an argument on the command line to hist.c.  The program outputs a
two dimensional "density of states" in short format (i.e. bins with zero contents
are omitted).  This density of states can be weighted to give the density
histogram at a different value if tau.

MM, 5.vii.02

..................................................................................

Version 5

Here, the restriction that all the histograms be measured at the same activity
has been lifted.  In the list specifying the location of the histograms, both
the value of tau and the activity must be specified after the filename.  The
resulting density of states must then by weighted in both activity and tau to
give the density histogram.

MM, 11.vii.02

..................................................................................

Version 6

Instead of storing all the histograms in memory in a three-dimensional matrix,
only bins with non-zero contents are stored.  This makes storage and retrieval
of elements more complicated, but means that more histograms can be combined
without running into memory limitations.

The utilities designed to work on the log_omegaNM.dat files produced with
version 5 can also be used here.

MM, 7.viii.02

..................................................................................

Version 7

The calculation of the matrix A and vector B has been made more efficient.  These
each require double sums over combinations of elements of the histograms.  The
effort can be reduced somewhat by making at least the outer loop over only the
non-zero elements of the histogram, since the compressed format gives us these
elements straight away.

Also, importantly, a bug has been fixed in the "extract" routine.  For this
reason alone, version 6 should not be used.

MM, 30.viii.02

..................................................................................

Version 8

This version reads the unnormalized raw histograms produced by stream2hist from
the output of sticky.c v19 or similar (i.e. NMsamp.xxx.dat files rather than
2Dhist.xxx.dat files).  The weight of each point is treated strictly as described
by Weerasinghe and Amar, unlike in previous versions, where the weighting was
only equivalent to W&A if all histograms were sampled in simulations of the same
length.

MM, 23.ix.02

................................................................................*/
 
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define E_N_BINS 1100

struct histogram {
   long n;     /* Number of particles */
   long m;     /* Number of binary contacts */
   long s;     /* Contents of bin */
};

struct histogram **data;   /* Histogram data in compressed format */
struct histogram ***slice; /* Pointers to slices in N of each histogram */
 
int main(int argc, char *argv[])
{
   char fn[200];      /* Filename of a histogram */
   char line[200];    /* A line of input */
   char **name;       /* Names of all the histograms */
   double **aa;       /* Matrix A in final linear equations */
   double activity;   /* Dummy variable for reading activity on first pass */
   double *bb;        /* Vector B in final linear equations */
   double omeganm;    /* Log of "2D density of states" */
   double sticky;     /* Dummy variable for reading tau on first pass */
   double *sumnm;     /* Histograms summed over N and M for each tau */
   double **sumi;     /* Histograms summed over tau for each N and M */
   double *tau;       /* Tau values of the histograms */
   double **v;        /* Matrix V in sing. val. decomp. A=U.W.V^transp */
   double volume;     /* Volume of system */
   double *w;         /* Diagonal elements of matrix W in sing. val. decomp. */
   double wmax;       /* Largest element of matrix W */
   double wmin;       /* Threshold below which elements of W are set to zero */
   double x;
   double *yy;        /* Log of Grand partition function, solved for in A.Y=B */
   double *z;         /* Activity of each histogram */
   int header;        /* 1 if the current N line in the output has been written */
   long i, j, k, p;
   long maxm;         /* Maximum number of binary contacts */
   long maxn;         /* Maximum number of particles */
   long mm;           /* M (number of binary contacts */
   long nn;           /* N (number of particles) */
   long nhist;        /* Number of 2D histograms */
   long *samples;     /* Number of bins with non-zero contents in each histogram */
   long s1, s2;
   FILE *fh;          /* Pointer to external file */

   long extract(long, long, long);
   void svbksb(double **, double [], double **, long, long, double [], double []);
   void svdcmp(double **, long, long, double [], double **);


   printf ("\nThis is version 8 of hist.c\n\n");


   /***** Check that the required arguments have been supplied *****/
   if (argc < 2) {
      printf ("Usage: %s <list>,\n", argv[0]);
      printf ("       where <list> is a file containing filenames of the 2D (N,M)\n");
      printf ("       histograms and the tau values at which they were measured\n");
      exit (1);
   }


   /***** Count the number of histograms *****/
   fh = fopen(argv[1], "r");
   if (fh == NULL) {
      fprintf (stderr, "ERROR: Could not open %s\n", argv[1]);
      exit (1);
   }
   nhist = 0;
   while (fgets(line, sizeof(line), fh)!=NULL &&
          sscanf(line, "%s %le %le", fn, &sticky, &activity)==3) nhist++;
   fclose (fh);
   printf ("%ld histograms found in %s\n", nhist, argv[1]);


   /***** Get names, taus and activities of all histograms *****/
   name = (char **)malloc((nhist+1)*sizeof(char *));
   for (i=0; i<=nhist; i++) name[i] = (char *)malloc(200*sizeof(char));
   tau = (double *)malloc((nhist+1)*sizeof(double));
   z = (double *)malloc((nhist+1)*sizeof(double));
   fh = fopen(argv[1], "r");
   if (fh == NULL) {
      fprintf (stderr, "ERROR: Could not open %s\n", argv[1]);
      exit (1);
   }
   for (i=1; i<=nhist; i++) {
      if (fgets(line, sizeof(line), fh)==NULL ||
          sscanf(line, "%s %le %le", name[i], &tau[i], &z[i])!=3) {
         fprintf (stderr, "ERROR: Could not read line %ld of %s\n", i, argv[1]);
         exit (1);
      }
   }
   fclose (fh);

 
   /***** Determine volume of system and hence maximum values of N and M from first histogram *****/
   fh = fopen(name[1], "r");
   if (fh == NULL) {
      fprintf (stderr, "ERROR: Could not open %s\n", name[1]);
      exit (1);
   }
   if (fgets(line, sizeof(line), fh) == NULL) {
      fprintf (stderr, "ERROR: Could not read first line of %s\n", name[1]);
      exit (1);
   }
   if (sscanf(line, "#volume %le", &volume) != 1) {
      fprintf (stderr, "ERROR: Could not get volume from first line of %s\n", name[1]);
      exit (1);
   }
   maxn = 1000;
   maxm = E_N_BINS;
   printf ("System volume: %.8le\n", volume);


   /***** Determine total number of bins with non-zero contents *****/
   samples = (long *)malloc( (nhist+1) * sizeof(long) );
   printf ("\n");
   for (i=1; i<=nhist; i++) {
      samples[i] = 0;
      fh = fopen(name[i], "r");
      if (fh == NULL) {
         fprintf (stderr, "ERROR: Could not open %s\n", name[i]);
         exit (1);
      }
      fgets(line, sizeof(line), fh);
      while (fgets(line, sizeof(line), fh) != NULL) {
         if (sscanf(line, "%ld %ld %ld", &nn, &mm, &s1) != 3) {
            fprintf (stderr,
               "ERROR: Could not read three integers from this line in %s:\n%s\n",
               name[i], line);
            exit (1);
         }
         samples[i]++;
      }
      fclose (fh);
      printf ("Non-zero bins in %s: %ld\n", name[i], samples[i]);
   }
   printf ("\n");


   /***** Read histograms and store in compressed format *****/
   /* "data" is a 2D array.  The first index specifies the index i, which labels
      the histogram number, i.e. a combination of tau and activity at which a
      histogram has been measured.  First allocate pointers to each of the
      histograms: */
   data = (struct histogram **)malloc( (nhist+1) * sizeof(struct histogram *) );
   /* Within each histogram, only bins with non-zero contents are stored.  This
      means that each histogram needs a different amount of memory and that
      each element must carry information about its (N,M) label as well as
      its contents, since the labels are not the same as the array index. */
   /* For each histogram we define an array of pointers to the start of each
      constant-N slice.  A null pointer indicates that no data were sampled for
      that value of N.  First define a pointer to the array of these arrays: */
   slice = (struct histogram ***)malloc( (nhist+1) * sizeof(struct histogram **) );
   for (i=1; i<=nhist; i++) {
      data[i] = (struct histogram *)malloc( (samples[i]+1) * sizeof(struct histogram) );
      slice[i] = (struct histogram **)malloc( (maxn+2) * sizeof(struct histogram *) );
      for (j=0; j<=maxn; j++) slice[i][j]=NULL;
      k = 0;
      fh = fopen(name[i], "r");
      if (fh == NULL) {
         fprintf (stderr, "ERROR: Could not open %s\n", name[i]);
         exit (1);
      }
      fgets(line, sizeof(line), fh);
      while (fgets(line, sizeof(line), fh) != NULL) {
         if (sscanf(line, "%ld %ld %ld", &nn, &mm, &s1) != 3) {
            fprintf (stderr,
               "ERROR: Could not read three integers from this line in %s:\n%s\n",
               name[i], line);
            exit (1);
         }
         data[i][k].n = nn;
         data[i][k].m = mm;
         data[i][k].s = s1;
         if (k==0 || nn != data[i][k-1].n) slice[i][nn+1]=&data[i][k];
         k++;
      }
      fclose (fh);
      if (k != samples[i]) {
         fprintf (stderr,
            "ERROR: Mismatch between original count of histogram %s and actual reading:\n",
            name[i]);
         fprintf (stderr, "       %ld versus %ld\n", samples[i], k);
         exit (1);
      }
      slice[i][nn+2] = &data[i][k];
      data[i][k].n = -1;
   }
   printf ("%ld histograms successfully read in compressed format\n\n", nhist);


   /***** Sums over histogram slices *****/
   sumnm = (double *)malloc( (nhist+1)*sizeof(double) );
   for (i=0; i<=nhist; i++) sumnm[i] = 0.0;
   sumi = (double **)malloc( (maxn+1)*sizeof(double *) );
   for (nn=0; nn<=maxn; nn++) {
      sumi[nn] = (double *)malloc( (maxm+1)*sizeof(double) );
      for (mm=0; mm<=maxm; mm++) sumi[nn][mm] = 0.0;
   }
   for (i=1; i<=nhist; i++) {
      for (j=0; j<samples[i]; j++) {
         sumnm[i] += (double)data[i][j].s;
         sumi[data[i][j].n+1][data[i][j].m+1] += (double)data[i][j].s;
      }
   }
   printf ("Sums of slices calculated\n");


   /***** Construct the matrix A *****/
   aa = (double **)malloc( (nhist+1)*sizeof(double *) );
   for (i=0; i<=nhist; i++) aa[i] = (double *)malloc( (nhist+1)*sizeof(double) );
   for (k=1; k<=nhist; k++) {
      for (j=1; j<=nhist; j++) {
         aa[k][j] = 0.0;
         if (k == j) aa[k][j] = 1.0;
         for (p=0; p<samples[k]; p++) {
            nn = data[k][p].n;
            mm = data[k][p].m;
            s1 = data[k][p].s;
            s2 = extract(j, nn, mm);
            if (s2 > 0) {
               aa[k][j] -= (double)s1 * (double)s2 / sumnm[k] / sumi[nn+1][mm+1];
            }
         }
      }
   }
   printf ("Matrix A constructed\n");


   /***** Construct the vector B *****/
   bb = (double *)malloc( (nhist+1)*sizeof(double) );
   for (k=1; k<=nhist; k++) {
      bb[k] = 0.0;
      for (p=0; p<samples[k]; p++) {
         nn = data[k][p].n;
         mm = data[k][p].m;
         s1 = data[k][p].s;
         x = log(sumnm[k]) - log((double)s1) + (mm*3.0+1.5)*tau[k] + nn*log(z[k]);
         for (j=1; j<=nhist; j++) {
            s2 = extract(j, nn, mm);
            if (s2 > 0) {
               x += (log((double)s2) - (mm*3.0+1.5)*tau[j] - nn*log(z[j]) - log(sumnm[j]))
                    * (double)s2 / sumi[nn+1][mm+1];
            }
         }
         x *= (double)s1 / sumnm[k];
         bb[k] += x;
      }
   }
   printf ("Vector B constructed\n");


   /***** Perform singular value decomposition *****/
   w = (double *)malloc( (nhist+1)*sizeof(double) );
   v = (double **)malloc( (nhist+1)*sizeof(double *) );
   for (j=0; j<=nhist; j++) v[j]=(double *)malloc( (nhist+1)*sizeof(double) );
   yy = (double *)malloc( (nhist+1)*sizeof(double) );
   svdcmp(aa, nhist, nhist, w, v);
   wmax = 0.0;
   for (j=1; j<=nhist; j++) if (w[j] > wmax) wmax=w[j];
   wmin = 1.0e-6 * wmax;
   for (j=1; j<=nhist; j++) if (w[j] < wmin) w[j]=0.0;
   svbksb(aa, w, v, nhist, nhist, bb, yy);
   printf ("Singular value decomposition completed\n");


   /***** Back substitute to get "density of states" and write it out on the fly *****/
   fh = fopen("log_omegaNM.dat", "w");
   for (nn=1; nn<=maxn; nn++) {
      header = 0;
      for (mm=1; mm<maxm; mm++) {
         if (sumi[nn][mm] > 0.0) {
            omeganm = 0.0;
            for (i=1; i<=nhist; i++) {
               s1 = extract(i, nn-1, mm-1);
               if (s1 > 0) {
                  omeganm += (log((double)s1) - ((mm-1)*3.0+1.5)*tau[i] - (nn-1)*log(z[i]) + yy[i] - log(sumnm[i]))
                           * (double)s1 / sumi[nn][mm];
               }
            }
            if (!header) {
               fprintf (fh, "N %ld %.14le\n", nn-1, (nn-1)/volume);
               header = 1;
            }
            fprintf (fh, "M %ld %.14le\n", mm-1, omeganm);
         }
      }
   }
   fclose (fh);
   printf ("\"Density of states\" written to log_omegaNM.dat\n\n");

   return 0;
}

/*................................................................................*/

/*
Retrieves contents of bin corresponding to nn particles and mm contacts from
histogram i.
*/

long extract(long i, long nn, long mm)
{
   struct histogram *a;

   if (slice[i][nn+1] == NULL) return 0.0;

   for (a=slice[i][nn+1]; (*a).n==nn; a++) {
      if ( (*a).m == mm ) return (*a).s;
      if ( (*a).m > mm ) return 0;
   }
   return 0.0;
}

/*................................................................................*/

#include <math.h>
#define NRANSI
#include "nrutil.h"

void svdcmp(double **a, long m, long n, double w[], double **v)
{
   double pythag(double a, double b);
   long flag,i,its,j,jj,k,l,nm;
   double anorm,c,f,g,h,s,scale,x,y,z,*rv1;

   rv1=vector(1,n);
   g=scale=anorm=0.0;
   for (i=1;i<=n;i++) {
      l=i+1;
      rv1[i]=scale*g;
      g=s=scale=0.0;
      if (i <= m) {
         for (k=i;k<=m;k++) scale += fabs(a[k][i]);
         if (scale) {
            for (k=i;k<=m;k++) {
               a[k][i] /= scale;
               s += a[k][i]*a[k][i];
            }
            f=a[i][i];
            g = -SIGN(sqrt(s),f);
            h=f*g-s;
            a[i][i]=f-g;
            for (j=l;j<=n;j++) {
               for (s=0.0,k=i;k<=m;k++) s += a[k][i]*a[k][j];
               f=s/h;
               for (k=i;k<=m;k++) a[k][j] += f*a[k][i];
            }
            for (k=i;k<=m;k++) a[k][i] *= scale;
         }
      }
      w[i]=scale *g;
      g=s=scale=0.0;
      if (i <= m && i != n) {
         for (k=l;k<=n;k++) scale += fabs(a[i][k]);
         if (scale) {
            for (k=l;k<=n;k++) {
               a[i][k] /= scale;
               s += a[i][k]*a[i][k];
            }
            f=a[i][l];
            g = -SIGN(sqrt(s),f);
            h=f*g-s;
            a[i][l]=f-g;
            for (k=l;k<=n;k++) rv1[k]=a[i][k]/h;
            for (j=l;j<=m;j++) {
               for (s=0.0,k=l;k<=n;k++) s += a[j][k]*a[i][k];
               for (k=l;k<=n;k++) a[j][k] += s*rv1[k];
            }
            for (k=l;k<=n;k++) a[i][k] *= scale;
         }
      }
      anorm=FMAX(anorm,(fabs(w[i])+fabs(rv1[i])));
   }
   for (i=n;i>=1;i--) {
      if (i < n) {
         if (g) {
            for (j=l;j<=n;j++)
               v[j][i]=(a[i][j]/a[i][l])/g;
            for (j=l;j<=n;j++) {
               for (s=0.0,k=l;k<=n;k++) s += a[i][k]*v[k][j];
               for (k=l;k<=n;k++) v[k][j] += s*v[k][i];
            }
         }
         for (j=l;j<=n;j++) v[i][j]=v[j][i]=0.0;
      }
      v[i][i]=1.0;
      g=rv1[i];
      l=i;
   }
   for (i=IMIN(m,n);i>=1;i--) {
      l=i+1;
      g=w[i];
      for (j=l;j<=n;j++) a[i][j]=0.0;
      if (g) {
         g=1.0/g;
         for (j=l;j<=n;j++) {
            for (s=0.0,k=l;k<=m;k++) s += a[k][i]*a[k][j];
            f=(s/a[i][i])*g;
            for (k=i;k<=m;k++) a[k][j] += f*a[k][i];
         }
         for (j=i;j<=m;j++) a[j][i] *= g;
      } else for (j=i;j<=m;j++) a[j][i]=0.0;
      ++a[i][i];
   }
   for (k=n;k>=1;k--) {
      for (its=1;its<=30;its++) {
         flag=1;
         for (l=k;l>=1;l--) {
            nm=l-1;
            if ((double)(fabs(rv1[l])+anorm) == anorm) {
               flag=0;
               break;
            }
            if ((double)(fabs(w[nm])+anorm) == anorm) break;
         }
         if (flag) {
            c=0.0;
            s=1.0;
            for (i=l;i<=k;i++) {
               f=s*rv1[i];
               rv1[i]=c*rv1[i];
               if ((double)(fabs(f)+anorm) == anorm) break;
               g=w[i];
               h=pythag(f,g);
               w[i]=h;
               h=1.0/h;
               c=g*h;
               s = -f*h;
               for (j=1;j<=m;j++) {
                  y=a[j][nm];
                  z=a[j][i];
                  a[j][nm]=y*c+z*s;
                  a[j][i]=z*c-y*s;
               }
            }
         }
         z=w[k];
         if (l == k) {
            if (z < 0.0) {
               w[k] = -z;
               for (j=1;j<=n;j++) v[j][k] = -v[j][k];
            }
            break;
         }
         if (its == 30) nrerror("no convergence in 30 svdcmp iterations");
         x=w[l];
         nm=k-1;
         y=w[nm];
         g=rv1[nm];
         h=rv1[k];
         f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
         g=pythag(f,1.0);
         f=((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
         c=s=1.0;
         for (j=l;j<=nm;j++) {
            i=j+1;
            g=rv1[i];
            y=w[i];
            h=s*g;
            g=c*g;
            z=pythag(f,h);
            rv1[j]=z;
            c=f/z;
            s=h/z;
            f=x*c+g*s;
            g = g*c-x*s;
            h=y*s;
            y *= c;
            for (jj=1;jj<=n;jj++) {
               x=v[jj][j];
               z=v[jj][i];
               v[jj][j]=x*c+z*s;
               v[jj][i]=z*c-x*s;
            }
            z=pythag(f,h);
            w[j]=z;
            if (z) {
               z=1.0/z;
               c=f*z;
               s=h*z;
            }
            f=c*g+s*y;
            x=c*y-s*g;
            for (jj=1;jj<=m;jj++) {
               y=a[jj][j];
               z=a[jj][i];
               a[jj][j]=y*c+z*s;
               a[jj][i]=z*c-y*s;
            }
         }
         rv1[l]=0.0;
         rv1[k]=f;
         w[k]=x;
      }
   }
   free_vector(rv1,1,n);
}
#undef NRANSI

/*................................................................................*/

/* note #undef's at end of file */
#define NRANSI
#include "nrutil.h"

void svbksb(double **u, double w[], double **v, long m, long n, double b[], double x[])
{
   long jj,j,i;
   double s,*tmp;

   tmp=vector(1,n);
   for (j=1;j<=n;j++) {
      s=0.0;
      if (w[j]) {
         for (i=1;i<=m;i++) s += u[i][j]*b[i];
         s /= w[j];
      }
      tmp[j]=s;
   }
   for (j=1;j<=n;j++) {
      s=0.0;
      for (jj=1;jj<=n;jj++) s += v[j][jj]*tmp[jj];
      x[j]=s;
   }
   free_vector(tmp,1,n);
}
#undef NRANSI

/*................................................................................*/

#include <math.h>
#define NRANSI
#include "nrutil.h"

double pythag(double a, double b)
{
   double absa,absb;
   absa=fabs(a);
   absb=fabs(b);
   if (absa > absb) return absa*sqrt(1.0+SQR(absb/absa));
   else return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+SQR(absa/absb)));
}
#undef NRANSI

/*................................................................................*/
