

/* This module contains functions for calculating the potential energy felt
 * by various sets of particles. Only pairwise interactions may be handled.
 * The form of the potential if defined by the global function pointers found
 * in potential.c */

/*globals*/
/*the maximum value of the imaginary temperature for bond orientation moves*/
double e_max_temp;
/*the minimum value of the imaginary temperature for bond orientation moves*/
double e_min_temp;

/*setters*/
void set_max_temp (double u) {
	e_max_temp = u;
}

void set_min_temp (double u) {
	e_min_temp = u;
}

void set_epsilon(double u) {
	p_set_epsilon(u);
}

double get_epsilon() {
	return p_get_epsilon();
}

/* read function, just passes to P_read and g_read*/
void e_read (FILE *infile){
	p_read(infile);
	g_read(infile);
}


/*FUNCTION: get_temp gets an imaginary temperature uniformly ditributed
 * between max_temp and min_temp
 *
 *OUTPUT: the temperature
 */
void get_temp() {
	temp = (e_max_temp - e_min_temp) * randomF() + e_min_temp; 
}

/*FUNCTION: energy_calc gets the potential energy of a single particle with
 * generalised coordinates u,v. The particle may be real (as in, existing in
 * the system) or imaginary (a test particle)
 *
 * Output: type double: The energy (relative to kT) of the test particle
 * Parameters:
 *	double coord_u[]: A list continaing the u coordinate of each particle
 *	in the system
 *	double coord_v[]: A list containing the v coordinate of each particle
 *	in the system
 *	double a_u: The u coordinate of the test particle
 *	double v_u: The v coordinate of the test particle
 */

double energy_calc(double coord_u[],double coord_v[], double a_u, double a_v,int index) {
	
	double energy =0;
	int i;

	//loop over all particles
	for (i = 0; i< index && i < n_particles; i++) {
		//printf("i:%d\n",i);
		energy += potential( distance_pot(a_u,a_v,coord_u[i],coord_v[i]  ) );
		//printf("%d %f: %f %f, %f %f\n",i,energy,a_u,a_v,coord_u[i],coord_v[i]);

	}

	for (i = index+1; i < n_particles; i++) {
		//printf("i:%d\n",i);
		energy += potential( distance_pot(a_u,a_v,coord_u[i],coord_v[i]  ) );
		//printf("%d %f: %f %f, %f %f\n",i,energy,a_v,a_v,coord_u[i],coord_v[i]);
	}

	//test for failure
	if (isnan(energy)) {
		 die ("energy_calc_error");
	}
	//return
	return energy;
}

/*FUNCTION: energy_calc_DA is used to calculate the (augmented,not real) energy of
 * a particle at u ,v. Using the data augmentation scheme, only particles in
 * different clusters are considered, and the imaginary temperature "temp" is used
 * to calculate the imaginary "bond energy"
 *
 * Output: type double: The energy (relative to kT) of the test particle
 * Parameters:
 *	double coord_u[]: A list continaing the u coordinate of each particle
 *	in the system
 *	double coord_v[]: A list containing the v coordinate of each particle
 *	in the system
 *	double temp: the imaginary temperature used to calculate the "energy" of
 *	the bond matrix
 *	double a_u: The u coordinate of the test particle
 *	double v_u: The v coordinate of the test particle
 *	int cluster_of[]: A list containing an index which uniquely identifies 
 *	the sluster each particle belongs to.
 */

double energy_calc_DA (double coord_u[], double coord_v[], double temp,int target, int cluster_of[]) {
	
	double energy =0;

	int i;
	double potentialV;
	double a_u = coord_u[target];
	double a_v = coord_v[target];
	int cl_of_target = cluster_of[target];

	//loop over all particles, check the cluster and then calculate the "energy"
	for (i = 0; i< n_particles; i++) {
		if (cluster_of[i] != cl_of_target) {
			potentialV = potential( distance_pot(a_u,a_v,coord_u[i],coord_v[i]  ) );
			energy += (potentialV - fmin(potentialV/temp,0.0));

		}
	}

	//check for errors
	if (isnan(energy)) die ("energy_calc_c error");
	
	//return
	return energy;
}

/*FUNCTION: energy_total is used to calculate the  energy of
 * the entire system in units of epsilon/kt
 *
 * Output: type double: The energy
 * Parameters:
 *	double coord_u[]: A list continaing the u coordinate of each particle
 *	in the system
 *	double coord_v[]: A list containing the v coordinate of each particle
 *	in the system
 */

double energy_total(double coord_u[], double coord_v[]) {

	int i;
	double energy = 0;
	for (i = 0; i<n_particles;i++) {
		energy += energy_calc(coord_u,coord_v,coord_u[i],coord_v[i],i);
	}
	return energy/p_get_epsilon();

}


/*
 *FUNCTION type int overlap:determines whether two particles overlap at all
 * Output: int: zero if no overlap, otherwise some non zero int
 *	double coord_u[]: A list continaing the u coordinate of each particle
 *	in the system
 *	double coord_v[]: A list containing the v coordinate of each particle
 *	in the system
 *	double temp: the imaginary temperature used to calculate the "energy" of
 *	the bond matrix
 *	double a_u: The u coordinate of the test particle
 *	double v_u: The v coordinate of the test particle
 */

int overlap (double coord_u[], double coord_v[], int target) {
	int i;

	for (i = 0; i < target; i++) {
		if (distance_NI(coord_u[target],coord_v[target],coord_u[i],coord_v[i]  ) < 1) {
				return 1;
		}
	}

	for (i = target+1; i < n_particles; i++) {
		if (distance_NI(coord_u[target],coord_v[target],coord_u[i],coord_v[i]  ) < 1) {
				return 1;
		}
	}	

	return 0;

}





