/*a module containing the basic geometrical functions on a rectangular flat plane
 * u: the x coordinate/xmax
 * v: the v coordinate/ymax
 *
 *the prefix for this module if fl_
 */


/*globals*/
double fl_x_max;
double fl_y_max;

/*read function for the plane*/
void fl_read (FILE *infile) {

	char command [50];
	char error[20];
	
	int successx = 0 ;
	int successy = 0 ;
	rewind(infile);
	/*read in values*/
	if (infile == NULL) die ("Could not open options file!");

	while (read_line(infile)) {
		get_string(command,sizeof(command));
		upper_case(command);

		if (*command =='#') {
			continue;
		} else if (strcmp(command,"XMAX")==0) {
			if (!get_double(&fl_x_max)) die ("Could not read max x");
			successx = 1;
		} else if (strcmp(command,"YMAX")==0) {
			if (!get_double(&fl_y_max)) die ("Could not read max y");
			successy = 1;
		}
	}

	if (successx == 0) die ("xmax not defined in options file");
	if (successy == 0) die ("ymax not defined in options file");

	if (fl_x_max < 0.99999) {
		printf("%f\n",fl_x_max);
		die("the system must be larger than a particle in each dimension!");
	}

	if (fl_y_max < 0.99999) {
		printf("%f\n",fl_y_max);
		die("the system must be larger than a particle in each dimension!");
	}	

}

/*FUNCTION: cartesian is a function which converts from the current
 * to cartesians.
 *
 * Output: double x,y,z: The cartesian coordinates of the particle
 * Input: coord_u, coord_v: The generalised coordinates of the particle
 */


void fl_cartesian(double *x, double *y, double *z,double coord_u, double coord_v) {

	*x = coord_u*fl_x_max;
	*y = coord_v*fl_y_max;
	*z = 0;

}

/*FUNCTION: cartesian_bx is a function which converts from the current
 * to cartesians, but shifts the origin so that no coordinates are negative.
 *
 * Output: double x,y,z: The cartesian coordinates of the particle
 * Input: coord_u, coord_v: The generalised coordinates of the particle
 */

void fl_cartesian_bx(double *x, double *y, double *z,double coord_u, double coord_v) {

	*x = coord_u*fl_x_max;
	*y = coord_v*fl_y_max;
	*z = 0;

}

/* FUNCTION: distance_NI returns the cartesian distance between two
 * particles. Under periodic bounderies, the nearest image convention is used.
 *
 * Output: double, The particle separation
 * Input double u1,v1,u2,v2: The generalised coordinates of the two particles*/

double fl_distance_NI (double u1,double v1, double u2, double v2){ 

	double dx = (u1-u2)*fl_x_max;
	if (dx < -0.5*fl_x_max ) {
		dx += fl_x_max;
	} else if (dx > 0.5*fl_x_max) {
		dx -= fl_x_max;
	}
	double dy = (v1-v2)*fl_y_max;
	if (dy < -0.5*fl_y_max ) {
		dy += fl_y_max;
	} else if (dy > 0.5*fl_y_max) {
		dy -= fl_y_max;
	}

	return sqrt(dx*dx+dy*dy);

}

/* FUNCTION: distance_NI_2 returns the square of the cartesian distance
 * between two particles. Faster than sqrt(distance_NI()).
 *
 * Output: double, The square of the particle separation
 * Input double u1,v1,u2,v2: The generalised coordinates of the two particles*/

double fl_distance_NI_square(double u1,double v1, double u2, double v2) {

	double dx = (u1-u2)*fl_x_max;
	if (dx < -0.5*fl_x_max ) {
		dx += fl_x_max;
	} else if (dx > 0.5*fl_x_max) {
		dx -= fl_x_max;
	}
	double dy = (v1-v2)*fl_y_max;
	if (dy < -0.5*fl_y_max ) {
		dy += fl_y_max;
	} else if (dy > 0.5*fl_y_max) {
		dy -= fl_y_max;
	}

	return dx*dx+dy*dy;

}

/* FUNCTION: uv_coords converts cartesian coordinates to generalised coordinates
 *
 * Input: double x,y,z the cartsian coordinates of the particle
 * Output: double *u,*v, pointers to a location to contain the generalised coordinates u,v
 */
void fl_uv_coords(double *u, double *v, double x, double y, double z) {

	*u = x/fl_x_max;
	*v = y/fl_y_max;

}






















